'**************************************************************************
'*           Long Windows 95 file names for QB/QBX/VBDOS programs         *
'*                              BASIC module                              *
'*                      (c) Hans Lunsing - 11/1996                        *
'**************************************************************************

' This source code is suitable for all Microsoft's Basic compilers, latest
' versions. However, the declaration in the precompiler if block
' #ifdef QB ... #endif is for QB only. Delete it for the other compilers.

DEFINT A-Z
'$INCLUDE: 'MBLFN.BI'

' Declare function SSEG for QB 4.x only
#ifdef QB
DECLARE FUNCTION SSEG(S AS STRING)
#endif

DECLARE SUB LfnAttrib (FileName AS STRING)
DECLARE SUB LfnFindClose ()
DECLARE SUB SetDTA (FileInfo AS DOSFileInfo)
DECLARE FUNCTION ToLNG& (Word AS INTEGER)

CONST CARRY = 1

DIM SHARED DOSError             AS INTEGER
DIM SHARED FileInfo             AS DOSFileInfo
DIM SHARED FindData             AS Win32FindData
DIM SHARED FindHandle           AS INTEGER
DIM SHARED REG                  AS RegTypeX

FUNCTION CurDrive$
'----------------------------------------------------------------------
' CurDrive$: Get current disk drive
'----------------------------------------------------------------------
REG.AX = &H1900
CALL INTERRUPTX(&H21, REG, REG)
CurDrive$ = CHR$((REG.AX AND &HFF) + 65)
END FUNCTION

FUNCTION DiskFree& (Drive AS STRING)
'----------------------------------------------------------------------
' Returns the number of free bytes on the given drive.
' If the given drive is invalid, the function returns 0, and DOSError
' has the value 15 (invalid disk drive).
'----------------------------------------------------------------------
DIM BytesInSector       AS LONG
DIM SectorsInCluster    AS LONG
DIM FreeClusters        AS LONG

REG.DX = GetDriveNumber(Drive)
REG.AX = &H3600
CALL INTERRUPTX(&H21, REG, REG)
IF REG.AX = -1 THEN
  DOSError = 15                         ' invalid disk drive
  DiskFree& = 0
ELSE
  DOSError = 0
  BytesInSector = ToLNG&(REG.CX)
  SectorsInCluster = ToLNG&(REG.AX)
  FreeClusters = ToLNG&(REG.BX)
  DiskFree& = FreeClusters * SectorsInCluster * BytesInSector
END IF
END FUNCTION

FUNCTION DOSErr%
'----------------------------------------------------------------------
' Returns the DOS error code
'----------------------------------------------------------------------
DOSErr% = DOSError
END FUNCTION

FUNCTION DOSErrTest%
'----------------------------------------------------------------------
' Returns the DOS error code, and resets it to zero
'----------------------------------------------------------------------
DOSErrTest% = DOSError
DOSError = 0
END FUNCTION

FUNCTION GetDriveNumber% (Drive AS STRING)
'----------------------------------------------------------------------
' Sets drive number for given drive:
' A or a = 1, B or b = 2 etc.
' If Drive is an empty string the drive number is 0 to designate the
' current drive. If the first character of Drive isn't a letter, the
' drive number is 255 to designate an invalid drive.
'----------------------------------------------------------------------
DIM DriveNumber AS INTEGER
IF LEN(Drive) THEN
  DriveNumber = ASC(Drive)
  SELECT CASE DriveNumber
  CASE 65 TO 90
    DriveNumber = DriveNumber - 64
  CASE 97 TO 122
    DriveNumber = DriveNumber - 96
  CASE ELSE
    DriveNumber = 255
  END SELECT
ELSE
  DriveNumber = 0
END IF
GetDriveNumber = DriveNumber
END FUNCTION

SUB GetFileInfo (FileData AS DOSFileInfo)
'----------------------------------------------------------------------
' Returns the DOS FileInfo structure
'----------------------------------------------------------------------
FileData = FileInfo
END SUB

SUB GetFindData (FileData AS Win32FindData)
'----------------------------------------------------------------------
' Returns the Win32 FindData structure
'----------------------------------------------------------------------
FileData = FindData
END SUB

SUB LfnAttrib (FileName AS STRING)
'----------------------------------------------------------------------
' Get or set attributes of a file.
' Called by LfnGetAttrib and LfnSetAttrib with REG.BX set to desired action,
' and REG.CX to file attributes if attributes must be set
'----------------------------------------------------------------------
DIM FileNameZ AS STRING
FileNameZ = FileName + CHR$(0)
IF LfnSupported THEN                    ' Long file names supported ?
  REG.AX = &H7143                       ' Win95 function get or set attributes
ELSE
  REG.AX = &H4300 + (REG.BX AND &HFF)   ' DOS function get or set attributes
END IF
REG.DS = SSEG(FileNameZ)                ' address of file name in DS:DX
REG.DX = SADD(FileNameZ)
CALL INTERRUPTX(&H21, REG, REG)         ' execute interrupt
IF (REG.FLAGS AND CARRY) THEN           ' if error
  DOSError = REG.AX                     ' error number in DOSError
  REG.CX = -1                           ' and REG.CX = -1 for LfnGetAttrib
ELSE
  DOSError = 0
END IF
END SUB

FUNCTION LfnCurDir$ (Drive AS STRING)
'----------------------------------------------------------------------
' Long file name replacement for QuickBasic's (PDS) CURDIR function.
' If an error is encountered the function returns "" as its value.
' Win95 returns the path all in caps.
'----------------------------------------------------------------------
DIM Buffer      AS STRING

IF LEN(Drive) = 0 THEN                  ' Empty drive name?
  Drive = CurDrive$                     ' then current drive
ELSE                                    ' else
  Drive = UCASE$(LEFT$(Drive$, 1))      ' get drive letter
END IF
REG.DX = GetDriveNumber(Drive)          ' and drive number
IF LfnSupported THEN                    ' Long file names supported ?
  BufLen = MAXPATH                      ' Buffer for long file name
  REG.AX = &H7147                       ' Win95 function get current dir
ELSE
  BufLen = MAXDOSPATH                   ' Buffer for 8.3 filename
  REG.AX = &H4700                       ' DOS function get current dir
END IF
Buffer = STRING$(BufLen + 1, 0)         ' create buffer (1 byte extra for zero)
REG.DS = SSEG(Buffer)                   ' get buffer address
REG.SI = SADD(Buffer)
CALL INTERRUPTX(&H21, REG, REG)         ' execute interrupt
IF (REG.FLAGS AND CARRY) THEN           ' if error
  DOSError = REG.AX                     ' error number in DOSError
  LfnCurDir$ = ""
ELSE
  DOSError = 0
  'the directory path:
  LfnCurDir$ = Drive + ":\" + LEFT$(Buffer, INSTR(Buffer, CHR$(0)) - 1)
END IF
END FUNCTION

FUNCTION LfnDirAlias$
'----------------------------------------------------------------------
' Returns the alternate filename (alias) after a call to LfnDirFirst or
' LfnDirNext has been made.
' On it self it doesn't return a meaningful result.
'----------------------------------------------------------------------
DIM Zero  AS INTEGER
IF LfnSupported AND FindHandle <> 0 THEN
  Zero = INSTR(FindData.cAlternateFileName, CHR$(0))
  LfnDirAlias$ = LEFT$(FindData.cAlternateFileName, Zero - 1)
ELSE
  LfnDirAlias$ = ""
END IF
END FUNCTION

FUNCTION LfnDirFirst$ (Mask AS STRING, Attrib AS INTEGER)
'----------------------------------------------------------------------
' Long file name replacement for QuickBasic's (PDS) DIR function, first time.
' If an error is encountered the function returns "" as its value.
'----------------------------------------------------------------------
DIM MaskZ AS STRING
DIM Zero  AS INTEGER

MaskZ = Mask + CHR$(0)

IF LfnSupported THEN

  ' Close a search handle not yet closed

  IF FindHandle THEN
    LfnFindClose
  END IF

  ' Clean the data structure

  FindData.cFileName = STRING$(MAXPATH, 0)
  FindData.cAlternateFileName = STRING$(14, 0)

  REG.AX = &H714E
  REG.CX = Attrib                               ' search attributes
  IF (REG.CX AND FILE.ATTRIBUTE.VOLUME) THEN    ' volume attribute included?
    ' only volume names match
    REG.CX = REG.CX + 256 * FILE.ATTRIBUTE.VOLUME
  ELSE
    ' include read-only files as does PowerBasic's DIR$,  and all files match
    REG.CX = (REG.CX OR FILE.ATTRIBUTE.READONLY) + 256 * FILE.ATTRIBUTE.NORMAL
  END IF
  REG.SI = 1                            ' MS-DOS date- and time format
  REG.ES = VARSEG(FindData)             ' Address of Win32FindData structure
  REG.DI = VARPTR(FindData)             ' receiving the file info
  REG.DS = SSEG(MaskZ)                  ' address of file mask
  REG.DX = SADD(MaskZ)
  CALL INTERRUPTX(&H21, REG, REG)
  IF (REG.FLAGS AND CARRY) THEN         ' if error
    DOSError = REG.AX                   ' error number in DOSError
    LfnFindClose                        ' close the find procedure
    LfnDirFirst$ = ""
  ELSE
    DOSError = 0
    FindHandle = REG.AX
    Zero = INSTR(FindData.cFileName, CHR$(0))
    LfnDirFirst$ = LEFT$(FindData.cFileName, Zero - 1)  ' the file name
  END IF

ELSE
  SetDTA FileInfo                       ' Set up FileInfo as DTA
  FileInfo.fiFileName = STRING$(13, 0)  ' Clean the data structure

  REG.AX = &H4E00
  ' search attributes including read-only files as does PowerBasic's DIR$
  REG.CX = Attrib OR FILE.ATTRIBUTE.READONLY
  REG.DS = SSEG(MaskZ)                  ' address of file mask
  REG.DX = SADD(MaskZ)
  CALL INTERRUPTX(&H21, REG, REG)
  IF (REG.FLAGS AND CARRY) THEN         ' if error
    DOSError = REG.AX                   ' error number in DOSError
    LfnDirFirst$ = ""
  ELSE
    DOSError = 0
    Zero = INSTR(FileInfo.fiFileName, CHR$(0))
    LfnDirFirst$ = LEFT$(FileInfo.fiFileName, Zero - 1)  ' the file name
  END IF
END IF

END FUNCTION

FUNCTION LfnDirNext$
'----------------------------------------------------------------------
' Long file name replacement for QuickBasic's (PDS) DIR function, next times.
' If an error is encountered the function returns "" as its value.
'----------------------------------------------------------------------
IF LfnSupported THEN
  ' Clean the data structure

  FindData.cFileName = STRING$(MAXPATH, 0)
  FindData.cAlternateFileName = STRING$(14, 0)

  REG.AX = &H714F
  REG.BX = FindHandle
  REG.SI = 1                            ' MS-DOS date- and time format
  REG.ES = VARSEG(FindData)             ' Address of Win32FindData structure
  REG.DI = VARPTR(FindData)             ' receiving the file info
  CALL INTERRUPTX(&H21, REG, REG)
  IF (REG.FLAGS AND CARRY) THEN         ' if error
    DOSError = REG.AX                   ' error number in DOSError
    LfnFindClose                        ' close the find procedure
    LfnDirNext$ = ""
  ELSE
    DOSError = 0
    Zero = INSTR(FindData.cFileName, CHR$(0))
    LfnDirNext$ = LEFT$(FindData.cFileName, Zero - 1)  ' the file name
  END IF
ELSE
  FileInfo.fiFileName = STRING$(13, 0)  ' Clean the data structure

  REG.AX = &H4F00
  CALL INTERRUPTX(&H21, REG, REG)
  IF (REG.FLAGS AND CARRY) THEN         ' if error
    DOSError = REG.AX                   ' error number in DOSError
    LfnDirNext$ = ""
  ELSE
    DOSError = 0
    Zero = INSTR(FileInfo.fiFileName, CHR$(0))
    LfnDirNext$ = LEFT$(FileInfo.fiFileName, Zero - 1)  ' the file name
  END IF

END IF
END FUNCTION

SUB LfnFiles
'----------------------------------------------------------------------
' Long file name replacement for QuickBasic's FILES statement
'----------------------------------------------------------------------
DIM col         AS INTEGER
DIM FileName    AS STRING

PRINT LfnCurDir$("")
FileName$ = LfnDirFirst$("*.*", &H10)
IF LfnSupported THEN
  DO WHILE LEN(FileName$)
    PRINT LfnDirAlias$;
    IF (LfnGetAttrib(FileName$) AND &H10) THEN
      PRINT TAB(14); "<DIR>";
    END IF
    PRINT TAB(22); FileName$
    FileName$ = LfnDirNext$
  LOOP
ELSE
  col = 1
  DO WHILE LEN(FileName$)
    IF col > 80 THEN
      PRINT
      col = 1
    END IF
    PRINT TAB(col); FileName$;
    IF (LfnGetAttrib(FileName$) AND &H10) THEN
      PRINT TAB(col + 13); "<DIR>";
    END IF
    col = col + 20
    FileName$ = LfnDirNext$
  LOOP
  PRINT
END IF
PRINT USING STRING$(10, "#"); DiskFree&("");
PRINT " Bytes free"

END SUB

SUB LfnFindClose
'----------------------------------------------------------------------
' Closes the long filename find procedure and frees its internal storage.
'----------------------------------------------------------------------
REG.AX = &H71A1
REG.BX = FindHandle
CALL INTERRUPTX(&H21, REG, REG)
END SUB

FUNCTION LfnGetAttrib (FileName AS STRING)
'----------------------------------------------------------------------
' Get attributes of a file.
' If an error is encountered the function returns -1 as its value.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------
REG.BX = 0                              ' Get attributes
LfnAttrib FileName
LfnGetAttrib = REG.CX
END FUNCTION

SUB LfnKill (FileName AS STRING)
'----------------------------------------------------------------------
' Long file name replacement for QuickBasic's KILL statement.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------
DIM FileNameZ   AS STRING

FileNameZ = FileName + CHR$(0)

IF LfnSupported THEN                    ' Long file names supported?
  REG.SI = 1                            ' Wildcards allowed
  REG.CX = FILE.ATTRIBUTE.NORMAL + 256 * FILE.ATTRIBUTE.NORMAL             ' search attrib's and attrib's to match
  REG.AX = &H7141                       ' Win95 function delete file
ELSE
  REG.AX = &H4100                       ' DOS function delete file
END IF
REG.DS = SSEG(FileNameZ)                ' address of file name
REG.DX = SADD(FileNameZ)
CALL INTERRUPTX(&H21, REG, REG)         ' execute interrupt
IF (REG.FLAGS AND CARRY) THEN           ' if error
  DOSError = REG.AX                     ' error number in DOSError
ELSE
  DOSError = 0
END IF
END SUB

SUB LfnName (OldName AS STRING, NewName AS STRING)
'----------------------------------------------------------------------
' Long file name replacement for QuickBasic's NAME statement.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------
DIM OldNameZ    AS STRING
DIM NewNameZ    AS STRING

OldNameZ = OldName + CHR$(0)            ' temporary ASCIIZ strings
NewNameZ = NewName + CHR$(0)

IF LfnSupported THEN                    ' if long file names supported
  REG.AX = &H7156                       ' Win95 rename file function
ELSE                                    ' otherwise
  REG.AX = &H5600                       ' DOS rename file function
END IF
REG.DS = SSEG(OldNameZ)                 ' address of old name in DS:DX
REG.ES = SSEG(NewNameZ)                 ' and of new name in ES:DI
REG.DX = SADD(OldNameZ)
REG.DI = SADD(NewNameZ)
CALL INTERRUPTX(&H21, REG, REG)         ' execute interrupt
IF (REG.FLAGS AND CARRY) THEN           ' if error
  DOSError = REG.AX                     ' error number in DOSError
ELSE
  DOSError = 0
END IF

END SUB

SUB LfnSetAttrib (FileName AS STRING, Attrib AS INTEGER)
'----------------------------------------------------------------------
' Set attributes of a file.
' Of course long file names can be used under Windows 95 only.
'----------------------------------------------------------------------
REG.BX = 1                              ' Set attributes
REG.CX = Attrib
LfnAttrib FileName
END SUB

SUB SetDTA (FileInfo AS DOSFileInfo)
'----------------------------------------------------------------------
' Sets up FileInfo as Disk Transfer Area
'----------------------------------------------------------------------
REG.AX = &H1A00
REG.DS = VARSEG(FileInfo)
REG.DX = VARPTR(FileInfo)
CALL INTERRUPTX(&H21, REG, REG)
END SUB

FUNCTION ToLNG& (Word AS INTEGER)
'----------------------------------------------------------------------
' Converts unsigned integer to long
'----------------------------------------------------------------------
IF Word < 0 THEN
  ToLNG& = Word + 65536
ELSE
  ToLNG& = Word
END IF
END FUNCTION

